# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:        modules/Imanager.ycp
# Package:     Configuration of imanager
# Summary:     Imanager settings, input and output functions
# Authors:     Mark Clark <mclark@novell.com>
#
# $Id: Imanager.ycp,v 1.1.2.40 2005/10/26 17:49:34 dlong Exp $
#
# Representation of the configuration of imanager.
# Input and output routines.
require "yast"

module Yast
  class ImanagerClass < Module
    def main
      textdomain "novell-imanager"

      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Summary"
      Yast.import "Popup"
      Yast.import "Edirectory"
      Yast.import "Mode"
      Yast.import "Stage"
      Yast.import "Service"
      Yast.import "NovellUtils"
      Yast.import "NovellValidators"
      Yast.import "NovellPatterns"
      Yast.import "NovellLDAP"
      Yast.import "EdirUtils"
      Yast.import "NovellAddonInstall"
      Yast.import "Arch"
      Yast.import "FileUtils"
      Yast.import "String"
      Yast.import "Wizard"
      @auto_clone = false

      # DSFW variables
      @xad_packages_installed = false
      @xadMode = false

      @xadTestMode = false

      # Settings Map
      @Settings = {}
      @oldSettings = {}

      # Data was modified?
      @modified = false


      @proposal_valid = false

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false
      @globals_configured = false

      # Are we in upgrade mode?
      @upgradeModeOES11SP2 = false # upgrading from OES 11 SP2
      @upgradeModeOES11SP3 = false # upgrading from OES 11 SP3
      @upgradeModeOES2015 = false # upgrading from OES 2015
      @upgradeModeOES2015SP1 = false # upgrading from OES 2015 SP1
      @upgradeModeOES2018 = false # upgrading from OES 2018
      @upgradeModeOES2018SP1 = false # upgrading from OES 2018 SP1
      @upgradeModeOES2018SP2 = false # upgrading from OES 2018 SP2

      # Are we in channel upgrade mode?
      @channelUpgradeMode = false


      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    # Read all imanager settings
    # @return true on success
    def Read
      Builtins.y2milestone("in Imanager::Read()")
      # Imanager read dialog caption
      caption = _("Initializing iManager Configuration")

      steps = 2

      sl = 500
      Builtins.sleep(sl)

      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          _("Read iManager settings"),
          _("Checking dependencies")
        ],
        [
          # Progress step 1/1
          _("Reading iManager settings..."),
          _("Checking dependencies"),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read current settings
      return false if Abort()
      Progress.NextStage
      # See if we are doing an upgrade from OES 2018 SP2
      @upgradeModeOES2018SP2 = NovellAddonInstall.IsProductUpgradeOES2018SP2(
        "imanager"
      )
      if @upgradeModeOES2018SP2 == false
        # See if we are doing an upgrade from OES 2018 SP1
        @upgradeModeOES2018SP1 = NovellAddonInstall.IsProductUpgradeOES2018SP1(
          "imanager"
        )
        if @upgradeModeOES2018SP1 == false
          # See if we are doing an upgrade from OES 2018
          @upgradeModeOES2018 = NovellAddonInstall.IsProductUpgradeOES2018(
            "imanager"
          )
          if @upgradeModeOES2018 == false
            # See if we are doing an upgrade from OES 2015 SP1
            @upgradeModeOES2015SP1 = NovellAddonInstall.IsProductUpgradeOES2015SP1(
              "imanager"
            )
            if @upgradeModeOES2015SP1 == false
              # See if we are doing an upgrade from OES 11 SP3
              @upgradeModeOES11SP3 = NovellAddonInstall.IsProductUpgradeOES11SP3(
                "imanager"
              )
              if @upgradeModeOES11SP3 == false
                # See if we are doing an upgrade from OES 2015
                @upgradeModeOES2015 = NovellAddonInstall.IsProductUpgradeOES2015(
                  "imanager"
                )
                if @upgradeModeOES2015 == false
                  # See if we are doing an upgrade from OES 11 SP2
                  @upgradeModeOES11SP2 = NovellAddonInstall.IsProductUpgradeOES11SP2(
                    "imanager"
                  )
                  if @upgradeModeOES11SP2
                    upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES11SP2(
                      "imanager"
                    )
                    @oldSettings = NovellAddonInstall.GetSysconfigMap(
                      Ops.get_string(upgradeInfo, "config_file", "")
                    )
                  end #upgrading from OES 2015
                else
                  upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2015(
                    "imanager"
                  )
                  @oldSettings = NovellAddonInstall.GetSysconfigMap(
                    Ops.get_string(upgradeInfo, "config_file", "")
                  )
                end #upgrading from OES 11SP3
              else
                upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES11SP3(
                  "imanager"
                )
                @oldSettings = NovellAddonInstall.GetSysconfigMap(
                  Ops.get_string(upgradeInfo, "config_file", "")
                )
              end
            else
              upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2015SP1(
                "imanager"
              )
              @oldSettings = NovellAddonInstall.GetSysconfigMap(
                Ops.get_string(upgradeInfo, "config_file", "")
              )
            end
          else
            upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2018(
              "imanager"
            )
            @oldSettings = NovellAddonInstall.GetSysconfigMap(
              Ops.get_string(upgradeInfo, "config_file", "")
            )
          end
        else
          upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2018SP1(
            "imanager"
          )
          @oldSettings = NovellAddonInstall.GetSysconfigMap(
            Ops.get_string(upgradeInfo, "config_file", "")
          )
        end
      else
        upgradeInfo = NovellAddonInstall.GetProductUpgradeInfoOES2018SP2(
          "imanager"
        )
        @oldSettings = NovellAddonInstall.GetSysconfigMap(
          Ops.get_string(upgradeInfo, "config_file", "")
        )
      end
    
      Builtins.y2milestone("imanager:oldSettings: %1", @oldSettings)

      # See if XAD (DSVW - Domain Services for Windows) is installed.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "Imanager:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end
      Builtins.y2milestone("Imanager:xadMode:%1", @xadMode)

      # If iManager is installed and the version is previous to 2.5 (1.5 or 2.0.2)
      # we do not want to allow this script to configure older versions of iManager.
      exec_cmd = "rpm -q novell-imanager"
      ret_val = Convert.to_integer(SCR.Execute(path(".target.bash"), exec_cmd))
      if ret_val == 0
        # note: - not sure if will use the getimanagerversion function.
        #string iman_version = "2.7";
        #iman_version = GetImanagerVersion();

        # Check for older versions of iManager, return if found.
        exec_cmd = "rpm -q novell-imanager-2.0.2"
        ret_val = Convert.to_integer(
          SCR.Execute(path(".target.bash"), exec_cmd)
        )
        if ret_val == 0
          Report.Error(
            _(
              "An older version of iManager has been detected.  This script will only configure iManager versions 2.5 and above.  Please use the install that was provided with the currently installed version."
            )
          )

          return false
        end

        exec_cmd = "rpm -q novell-imanager-1.5"
        ret_val = Convert.to_integer(
          SCR.Execute(path(".target.bash"), exec_cmd)
        )
        if ret_val == 0
          Report.Error(
            _(
              "An older version of iManager has been detected.  This script will only configure iManager versions 2.5 and above.  Please use the install that was provided with the currently installed version."
            )
          )

          return false
        end
      end

      # determine if iManager configuration already exists
      cfile_exists = true
      service_configured = Convert.to_string(
        SCR.Read(path(".sysconfig.imanager.SERVICE_CONFIGURED"))
      )
      service_configured = Builtins.tolower(service_configured)
      if service_configured != "yes"
        cfile_exists = false
      end

      if @upgradeModeOES11SP2 || @upgradeModeOES2015 || @upgradeModeOES2015SP1 || @upgradeModeOES11SP3 || @upgradeModeOES2018 || @upgradeModeOES2018SP1 ||  @upgradeModeOES2018SP2
        Ops.set(
          @Settings,
          "CONFIG_IMANAGER_INSTALL_PLUGINS",
          Ops.get_string(@oldSettings, "CONFIG_IMANAGER_INSTALL_PLUGINS", "")
        )
      else
        Ops.set(
          @Settings,
          "CONFIG_IMANAGER_INSTALL_PLUGINS",
          Convert.to_string(
            SCR.Read(path(".sysconfig.imanager.CONFIG_IMANAGER_INSTALL_PLUGINS"))
          )
        )
      end

      # First, get the default LDAP/eDir data
      progress_orig = Progress.set(false)
      NovellLDAP.Read
      Progress.set(progress_orig)

      tmpstr = nil
      if cfile_exists
        if Mode.normal
          name = "iManager"
          if !Popup.YesNo(
              Builtins.sformat(
                _(
                  "It appears that %1 has already been configured.  To reconfigure %2 please\n" +
                    "exit and use iManager. If %3 has been uninstalled and is being reinstalled you may\n" +
                    "continue.  Continue configuration?"
                ),
                name,
                name,
                name
              )
            )
            return false
          end
        end
      end

      # Error message
      Report.Error(_("Cannot read current iManager settings.")) if false
      Builtins.sleep(sl)

      # check dependencies
      return false if Abort()
      Progress.NextStage

      @globals_configured = true
      return false if Abort()
      Progress.NextStage
      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      return false if Abort()
      @modified = false
      true
    end

    # Remove the ZeroG registry file if it exists
    # @return success
    def RemoveZerogRegistry
      bVal = false

      if FileUtils.Exists("/var/.com.zerog.registry.xml")
        # Try to delete the ZeroG (InstallAnywhere registry file)
        registry = "/var/.com.zerog.registry.xml"
        bVal = SCR.Execute(
          path(".target.bash"),
          Ops.add("/bin/rm -f ", registry)
        ) == 0
        if bVal == true
          Builtins.y2milestone(
            "removed old InstallAnywhere registry file: %1",
            registry
          )
        end
      end

      bVal
    end

    #    The installNPMs function will scan the npm dir and look for
    #    for any npm files.  If any npm files are found, the function
    #    will make a call to the java npm installer for each file.
    def installNPMs(installFlag)
      _WEB_INF_HOME = "/var/opt/novell/iManager/nps/WEB-INF"
      _BINPATH = "/var/opt/novell/iManager/nps/WEB-INF/bin"
      _LIBPATH = "/var/opt/novell/iManager/nps/WEB-INF/lib"
      #string NOVELL_TOMCAT_HOME= "/var/opt/novell/tomcat";
      _NOVELL_TOMCAT_HOME = "/var/opt/novell/tomcat"
      _NPM_DIR = "/var/opt/novell/iManager/nps/packages/"

      # Use the default jvm if the OS is 64 bit. There doesn't appear to be a problem with IA 7 and the ibm 1.5 64 bit jvm
      # when running on a 64 bit OS.
      # If the OS is 32 bit use either the sun or ibm 1.4.2 jvm. If neither 1.4.2 package is found,
      # revert back to the old JAVA_CMD path.
      # iManager requires a 1.4.2 jvm when running on a 32 bit OS in order to work around an ibm 1.5 and
      # InstallAnywhere 7 problem which causes the plug-in install randomly not install plug-ins or hang.

      # Use this Arch function to determine the correct java lib|lib64 path.
      #string lib = "lib";  // this is the 32 bit version
      #lib = "lib64";   // this is the 64 bit path

      #string JAVA_CMD = "/usr/lib/jvm/java-1_4_2-sun-1.4.2.13/jre/bin/java -Xms64m -Xmx96m";
      _JAVA_CMD = "/usr/bin/java -Xms64m -Xmx96m" # Default to the ibm 1.5 jvm

      # Todo: combine the sun and ibm blocks after debugging and testing these changes. Remove redundant JAVA_CMD lines
      # Use ls command to get correct java path.
      # map lsout = (map)SCR::Execute(.target.bash_output, "/bin/ls /usr/lib/jvm/java-1_4_2-sun*/jre/bin/java");
      java142_path = ""
      lsout = {}

      #if(lsout["exit"]:-1 == 0)
      #{
      #    java142_path = (lsout["stdout"]:"");
      #    java142_path = String::Replace(java142_path, "\n", "");
      #    y2milestone("iManager:installNPMs: ls command sun java142 path output: %1", java142_path);
      # }
      # Temp fix : added java-1_4_2-ibm instead of java-1_4_2-sun  to skip 1_4_2 java during installation of npms
      exec_cmd = "rpm  -q java-1_4_2-ibm"
      ret_val = Convert.to_integer(SCR.Execute(path(".target.bash"), exec_cmd))
      if ret_val == 0
        lsout = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            "/bin/ls /usr/lib/jvm/java-1_4_2-sun*/jre/bin/java"
          )
        )
        if Ops.get_integer(lsout, "exit", -1) == 0
          java142_path = Ops.get_string(lsout, "stdout", "")
          java142_path = String.Replace(java142_path, "\n", "")
          Builtins.y2milestone(
            "iManager:installNPMs: ls command sun java142 path output: %1",
            java142_path
          )
        end
        # Get the correct sun jvm path. If it isn't installed, use the default ibm 1.5 from /usr/bin/java
        # JAVA_CMD = "/usr/lib/jvm/java-1_4_2-sun-1.4.2.13/jre/bin/java -Xms64m -Xmx96m";
        if java142_path != ""
          _JAVA_CMD = Ops.add(java142_path, " -Xms64m -Xmx96m")
          Builtins.y2milestone(
            "iManager:installNPMs: Setting the JAVA_CMD to the sun path of : %1",
            _JAVA_CMD
          )
        else
          _JAVA_CMD = "/usr/bin/java -Xms64m -Xmx96m"
        end
      else
        # Check for the ibm 1.4.2 jvm next. If it isn't installed, use the default ibm 1.5 from /usr/bin/java
        exec_cmd2 = "rpm  -q java-1_4_2-ibm"
        ret_val2 = Convert.to_integer(
          SCR.Execute(path(".target.bash"), exec_cmd2)
        )
        if ret_val2 == 0
          # JAVA_CMD = "/usr/lib/jvm/java-1_4_2-ibm-1.4.2_sr7/jre/bin/java -Xms64m -Xmx96m";
          lsout = Convert.to_map(
            SCR.Execute(
              path(".target.bash_output"),
              "/bin/ls /usr/lib/jvm/java-1_4_2-ibm*/jre/bin/java"
            )
          )
          if Ops.get_integer(lsout, "exit", -1) == 0
            java142_path = Ops.get_string(lsout, "stdout", "")
            java142_path = String.Replace(java142_path, "\n", "")
            Builtins.y2milestone(
              "iManager:installNPMs: ls command ibm java142 path output: %1",
              java142_path
            )
          end
          if java142_path != ""
            _JAVA_CMD = Ops.add(java142_path, " -Xms64m -Xmx96m")
            Builtins.y2milestone(
              "iManager:installNPMs: Setting the JAVA_CMD to the ibm path of : %1",
              _JAVA_CMD
            )
          else
            _JAVA_CMD = "/usr/bin/java -Xms64m -Xmx96m"
          end
        else
          _JAVA_CMD = "/usr/bin/java -Xms64m -Xmx96m"
        end
      end

      _CLASSPATH = Ops.add(
        _NOVELL_TOMCAT_HOME,
        "/lib/tomcat-servlet-3.1-api.jar"
      )

      _CLASSPATH = Ops.add(
        _CLASSPATH,
        ":/var/opt/novell/iManager/nps/WEB-INF/classes"
      )
      _CLASSPATH = Ops.add(
        Ops.add(Ops.add(_CLASSPATH, ":"), _LIBPATH),
        "/eMFrame.jar"
      )
      _CLASSPATH = Ops.add(
        Ops.add(Ops.add(_CLASSPATH, ":"), _LIBPATH),
        "/AdminNamespace.jar"
      )
      _CLASSPATH = Ops.add(
        Ops.add(Ops.add(_CLASSPATH, ":"), _LIBPATH),
        "/rbsCore.jar"
      )
       _CLASSPATH = Ops.add(
        Ops.add(Ops.add(_CLASSPATH, ":"), _LIBPATH),
        "/encoder-1.2.1.jar"
      )
      _CLASSPATH = Ops.add(
        Ops.add(Ops.add(_CLASSPATH, ":"), _BINPATH),
        "/iManagerModuleInstall.jar"
      )
      # Added new class name for oes-cypress.

      _CLASSNAME = "com.novell.application.iManager.install.InstallNPMsOnLinux_OES"

      npmPath = "/var/opt/novell/iManager/nps/packages"

      # get all files in the npm directory
      files = Convert.convert(
        SCR.Read(path(".target.dir"), npmPath),
        :from => "any",
        :to   => "list <string>"
      )

      # keep only files that end with .npm
      npmFiles = Builtins.filter(files) do |ext|
        Builtins.issubstring(ext, "npm")
      end
      #     Excluding Quickfinder and Archive Versioning imanager plugins as products are removed from OES2015
      npmFiles = Builtins.filter(npmFiles) do |ext|
        !Builtins.issubstring(ext, "QFSearch")
      end
      npmFiles = Builtins.filter(npmFiles) do |ext|
        !Builtins.issubstring(ext, "arkmgmt")
      end

      #npmFiles = sort( npmFiles );
      npmFiles = Builtins.sort(npmFiles) do |x, y|
        Ops.greater_than(Builtins.tolower(x), Builtins.tolower(y))
      end

      baseNpmFiles = Builtins.filter(npmFiles) do |ext|
        Builtins.issubstring(ext, "base")
      end
      iManNpmFiles = Builtins.filter(npmFiles) do |ext|
        Builtins.issubstring(ext, "iManager")
      end
      coreNpmFiles = Convert.convert(
        Builtins.merge(baseNpmFiles, iManNpmFiles),
        :from => "list",
        :to   => "list <string>"
      )
      npmFiles = Builtins.sort(npmFiles) do |x, y|
        Ops.less_than(Builtins.tolower(x), Builtins.tolower(y))
      end
      npmFiles = Convert.convert(
        Builtins.union(coreNpmFiles, npmFiles),
        :from => "list",
        :to   => "list <string>"
      )

      # THIS IS TO ENSURE THAT FIELD PATCHES GET INSTALLED LAST
      fieldPatches = Builtins.filter(npmFiles) do |ext|
        Builtins.issubstring(ext, "iMan2")
      end

      # filter out the field patches
      npmFiles = Builtins.filter(npmFiles) do |ext|
        !Builtins.issubstring(ext, "iMan2")
      end
      npmFiles = Convert.convert(
        Builtins.merge(npmFiles, fieldPatches),
        :from => "list",
        :to   => "list <string>"
      )

      if npmFiles == nil
        Builtins.y2milestone("iManager:installNPMs:Found no npm files")
        return false
      else
        Builtins.y2milestone("iManager:installNPMs:Found files:\n%1", npmFiles)
        main_cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(_JAVA_CMD, " -Dcom.ibm.tools.attach.enable=no"),
                    " -cp "
                  ),
                  _CLASSPATH
                ),
                " "
              ),
              _CLASSNAME
            ),
            " "
          ),
          installFlag
        )
        execute_cmd = ""
        rval = -1
        retval = {}
        Builtins.foreach(npmFiles) do |nFile|
          execute_cmd = Ops.add(
            Ops.add(Ops.add(main_cmd, " "), _NPM_DIR),
            nFile
          )
          Builtins.y2milestone("iManager:installNPMs:cmd is: %1", execute_cmd)
          #rval = (integer)SCR::Execute(.target.bash, execute_cmd);
          retval = Convert.to_map(
            SCR.Execute(path(".target.bash_output"), execute_cmd)
          )
          Builtins.y2milestone(
            "imanager:installNPMs:output:Map Value:\n%1",
            retval
          )
          if Ops.get_integer(retval, "exit", -1) != 0
            Builtins.y2error("Warning: Failed to install npm %1", nFile)
            Report.Error("Failed to install npm")
          else
            Builtins.y2milestone(
              "iManager:installNPMs:cmd succeeded: %1",
              execute_cmd
            )
          end #if(rval != 0)
          #Report::Warning(_("Unable to install npm file: ") + nFile);
        end

        # test - From Storage.ycp
        #  string cmd = sformat("ntfsresize -f -i %1", device );
        #  y2milestone( "GetFreeSpace Executing cmd:%1", cmd );
        #  map bcall = (map) SCR::Execute( .target.bash_output, cmd,
        #                                  $[ "LC_MESSAGES" :"POSIX"] );
        #  y2milestone( "GetFreeSpace Executing ret:%1", bcall );
        #  string tmp = _("Resize Not Possible:") + "\n\n";
        #  tmp = tmp + bcall["stdout"]:"" + bcall["stderr"]:"";
        #  Popup::Error( tmp );
        #  return( $[] );
        # End test

        # Fix for bug 225129 - unable to remove plug-ins. Because the logged in user does not have file permissions to remove the
        # InstallAnywhere registry file when iManager is running we will remove it here after the installation of plug-ins is finished.
        Builtins.y2milestone(
          "iManager:installNPMs:Removing InstallAnywhere registry file."
        )
        RemoveZerogRegistry()
      end

      true
    end

    #   The uninstall function should remove all of the old iManager 2.0.2 packages from the rpm database
    #   as well as delete the old iManager directories
    def uninstall_iman202
      execute_cmd = ""

      # Remove the old iManager 2.0.2 packages
      rpmList = []

      Builtins.add(rpmList, "novell-arkmanager-imanager")
      Builtins.add(rpmList, "novell-cluster-services-imanager")
      Builtins.add(rpmList, "novell-nss-imanager")
      Builtins.add(rpmList, "novell-plugin-wanman")
      Builtins.add(rpmList, "novell-plugin-service-manager")
      Builtins.add(rpmList, "novell-plugin-snmp")
      Builtins.add(rpmList, "novell-plugin-rwiz")
      Builtins.add(rpmList, "novell-plugin-repair")
      Builtins.add(rpmList, "novell-plugin-pki")
      Builtins.add(rpmList, "novell-plugin-nmas")
      Builtins.add(rpmList, "novell-plugin-merge")
      Builtins.add(rpmList, "novell-plugin-ldap")
      Builtins.add(rpmList, "novell-plugin-indexmanager")
      Builtins.add(rpmList, "novell-plugin-ice")
      Builtins.add(rpmList, "novell-plugin-backup-restore")
      Builtins.add(rpmList, "novell-netstorage-imanager-plugin")
      Builtins.add(rpmList, "novell-usermanagement-imanager-plugin")
      Builtins.add(rpmList, "novell-iprint-management")
      Builtins.add(rpmList, "novell-imanager")

      rval = -1
      Builtins.foreach(rpmList) do |pkgName|
        execute_cmd = Ops.add("rpm -q ", pkgName)
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval == 0
          execute_cmd = Ops.add("rpm -e ", pkgName)
          rval = Convert.to_integer(
            SCR.Execute(path(".target.bash"), execute_cmd)
          )
          if rval != 0
            Report.Warning(Ops.add(_("Unable to remove package: "), pkgName))
          end
        end
      end

      # Check to see if eDir 8.7.3 is installed. If not installed remove everything we put down
      # If eDir 8.7.3 is installed don't remove any of the shared libraaries
      # Returns 0 if true, non-zero if false
      edir_rpms202 = []
      execute_cmd = "rpm -qa|grep -i NDSserv"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      if rval == 0
        Builtins.add(edir_rpms202, "ntls")
        Builtins.add(edir_rpms202, "NOVLpkit")
        Builtins.add(edir_rpms202, "NOVLpkia")
        Builtins.add(edir_rpms202, "NOVLice")
        Builtins.add(edir_rpms202, "NOVLlmgnt")
        Builtins.add(edir_rpms202, "NOVLxis")
        Builtins.add(edir_rpms202, "NLDAPbase")
        Builtins.add(edir_rpms202, "NLDAPsdk")
        Builtins.add(edir_rpms202, "NDSbase")
        Builtins.add(edir_rpms202, "nici")
      end

      Builtins.foreach(edir_rpms202) do |pkgName|
        execute_cmd = Ops.add("rpm -q ", pkgName)
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval == 0
          execute_cmd = Ops.add("rpm -e ", pkgName)
          rval = Convert.to_integer(
            SCR.Execute(path(".target.bash"), execute_cmd)
          )
          if rval != 0
            Report.Warning(Ops.add(_("Unable to remove package: "), pkgName))
          end
        end
      end

      true
    end

    # Configure the SuSEFirewall with the appropriate ports
    #
    # @return [Boolean] true on success
    def ConfigureFirewall
      retVal = NovellUtils.SetFirewallPort(80, :tcp, true)
      retVal = retVal && NovellUtils.SetFirewallPort(443, :tcp, true)
      retVal
    end


    # Write all imanager settings
    # @return true on success
    def Write
      Builtins.y2milestone("Imanager::Write started")
      Wizard.DisableAbortButton

      pattern = "novell-imanager"
      if NovellPatterns.IsPatternInstalled(pattern) == false
        Builtins.y2milestone(
          "The required pattern, %1, is not installed.",
          pattern
        )
        if Mode.normal
          Report.Error(
            Builtins.sformat(
              _(
                "Configuration aborted. Install the required %1 pattern and try again."
              ),
              pattern
            )
          )
        end

        return true
      end
      if !Ops.get(NovellAddonInstall.products_to_configure, "imanager", true)
        Builtins.y2milestone("iManager -- configure later")
        return true
      end

      abortInstall = Builtins.tolower(
        NovellUtils.ReadOption("ABORT_OES_INSTALL")
      )
      if abortInstall == "yes"
        Builtins.y2milestone(
          "iManager:setting ABORT_OES_INSTALL is set to 'yes'."
        )
        return true
      end
      # See if XAD (DSVW - Domain Services for Windows) is installed.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )

      @xad_packages_installed = true if @xadTestMode

      Builtins.y2milestone(
        "Imanager:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end
      Builtins.y2milestone("Imanager:xadMode:%1", @xadMode)

      # Imanager read dialog caption
      caption = _("Completing iManager Configuration")

      steps = 2

      sl = 500

      Builtins.sleep(sl)

      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/2
          _("Configure iManager"),
          # Progress stage 2/2
          _("Install iManager Plug-ins")
        ],
        [
          # Progress step 1/2
          _("Configuring iManager..."),
          # Progress step 2/2
          _("Installing iManager Plug-ins..."),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # write settings
      return false if Abort()
      Progress.NextStage

      # Since this configuration manipulates www:wwwrun which can be LUM enabled, we will start namcd here for upgrades.
      if @upgradeModeOES11SP2 || @upgradeModeOES2015 || @upgradeModeOES2015SP1 || @upgradeModeOES11SP3 || @upgradeModeOES2018 || @upgradeModeOES2018SP1 ||  @upgradeModeOES2018SP2
        Builtins.y2milestone("imanager.Write:starting namcd for upgrade.")
        Service.Restart("namcd")
        Service.Restart("sshd")

        Builtins.sleep(10000) #sleep 10 seconds
      end

      # If iManager is installed and the version does not equal 2.5 or greater,
      # we do not want to allow this script to configure other versions of iManager.
      exec_cmd = "rpm -q novell-imanager"
      ret_val = Convert.to_integer(SCR.Execute(path(".target.bash"), exec_cmd))
      if ret_val == 0
        # Check for older versions of iManager, return if found.
        exec_cmd = "rpm -q novell-imanager-2.0.2"
        ret_val = Convert.to_integer(
          SCR.Execute(path(".target.bash"), exec_cmd)
        )
        if ret_val == 0
          Report.Error(
            _(
              "An older version of iManager has been detected.  This script will only configure iManager versions 2.5 and above.  Please use the install that was provided with the currently installed version."
            )
          )

          return false
        end

        exec_cmd = "rpm -q novell-imanager-1.5"
        ret_val = Convert.to_integer(
          SCR.Execute(path(".target.bash"), exec_cmd)
        )
        if ret_val == 0
          Report.Error(
            _(
              "An older version of iManager has been detected.  This script will only configure iManager versions 2.5 and above.  Please use the install that was provided with the currently installed version."
            )
          )

          return false
        end 

        #exec_cmd = "rpm -q novell-imanager-2.5";
        #ret_val = (integer)SCR::Execute(.target.bash, exec_cmd);
        #if(ret_val != 0)
        #{
        #  Report::Error (_("Another version of iManager has been detected.  This script will only configure
        #iManager 2.5.  Please use the install that was provided with the currently installed version."));
        #        return false;
        #      }
      end

      # Write the /etc/sysconfig file
      SCR.Write(path(".sysconfig.imanager.SERVICE_CONFIGURED"), "no")

      # The following will be a one-to-one translation from the iManager python code to YaST.

      treeName = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_TREE_NAME",
        ""
      )
      userName = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        ""
      )

      if @xadMode
        xad_tree_admin_context = Ops.get_string(
          NovellLDAP.Settings,
          "XAD_TREE_ADMIN_CONTEXT",
          ""
        )
        Builtins.y2milestone(
          "Imanager:XAD_TREE_ADMIN_CONTEXT:%1",
          xad_tree_admin_context
        )
        if xad_tree_admin_context != nil && xad_tree_admin_context != ""
          userName = NovellUtils.Nds2Ldap(xad_tree_admin_context)
        end
      end
      Builtins.y2milestone("Imanager:admin:%1", userName)

      if !@upgradeModeOES11SP2 && !@upgradeModeOES2015 && !@upgradeModeOES2015SP1 && !@upgradeModeOES11SP3 && !@upgradeModeOES2018 && !@upgradeModeOES2018SP1 && !@upgradeModeOES2018SP2
        simpleUserName = NovellUtils.Typeful2Typeless(userName)
        iman_conf = Ops.add(
          Ops.add(Ops.add(simpleUserName, "."), Builtins.toupper(treeName)),
          "=eDirectory"
        )
  
        # The ldap treeName and userName are optional. Make sure something was entered before creating the configiman.properties file. An empty configiman.properties file is as bad as one with garbage in it.
        if treeName != "" && userName != ""
          # Start out with a blank file.
          SCR.Execute(
            path(".target.mkdir"),
            "/var/opt/novell/iManager/nps/WEB-INF"
          )
          bval = SCR.Write(
            path(".target.string"),
            "/var/opt/novell/iManager/nps/WEB-INF/configiman.properties",
            ""
          )
  
          bval = SCR.Write(
            path(".target.string"),
            "/var/opt/novell/iManager/nps/WEB-INF/configiman.properties",
            iman_conf
          )
  
          if bval != true
            Report.Warning(
              _(
                "Can not open configiman.properties for writing.  Please refer to the documentation for instructions on creating this file."
              )
            )
          end
        end
      end

      # First implement the init method - we have to implement everything, because they want to handle the upgrade case.
      upgrade_iman = "no"
      iman_version = "2.5.0"

      execute_cmd = "rpm -qa |grep novell-imanager-2.0."
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      if rval == 0
        _DEX_NPM_PATH = "/var/opt/novell/iManager/nps/packages/StandardGadgets.npm"
        _VO_NPM_PATH = "/var/opt/novell/iManager/nps/packages/VirtualOffice.npm"

        # If there is an old version of iManager, check for Dex and Virtual Office and set the upgrade variable
        upgrade_iman = "yes"

        if SCR.Read(path(".target.size"), _DEX_NPM_PATH) != -1
          Report.Error(
            _(
              "iManager 2.5 cannot be installed if DEX is installed. Please uninstall DEX and run the iManager install again."
            )
          )
          return false
        end

        if SCR.Read(path(".target.size"), _VO_NPM_PATH) != -1
          Report.Error(
            _(
              "iManager 2.5 cannot be installed if Virtual Office 1.0 is installed. Please uninstall Virtual Office and run the iManager install again."
            )
          )
          return false
        end

        execute_cmd = "rpm -q NOVLiMgr"
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval == 0
          errorMsg = "A previous version of iManager (1.5.x) has been detected on your " +
            "system. You must uninstall this version and stop the TOMCAT " +
            "service it was using before running this install.\n" +
            "The uninstall for iManager version 1.5.x is located in: " +
            "TOMCAT_HOME/webapps/eMFrame/UninstallerData"

          Report.Error(errorMsg)
          return false
        end
      end

      # Now, we will implement the pre_install method of the python code which does the upgrade.
      # This could have been done above, but we will keep it this way to match the python.
      if upgrade_iman == "yes"
        # Upgrade Steps
        # 1- Check for Dex and VO (done in the init function)
        # 2- Remove old 2.0.2 packages
        # 3- Remove old nps_2.0.2 directory if one already exists
        # 4- Move nps directory to /tomcat4/nps_<version>
        # 5- Copy back the old npms and the custom content

        execute_cmd = "rpm -q novell-imanager-2.0.2"
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval == 0
          iman_version = "2.0.2"
        else
          iman_version = "2.0.1"
        end

        # 2- Remove old 2.0.2 packages
        uninstall_iman202

        # 3- Remove old nps_2.0.2 direcrory
        targetdir = Ops.add("/var/opt/novell/tomcat4/nps_", iman_version)
        if SCR.Read(path(".target.dir"), targetdir) != nil
          execute_cmd = Ops.add(
            "rm -rf /var/opt/novell/tomcat4/nps_",
            iman_version
          )
          rval = Convert.to_integer(
            SCR.Execute(path(".target.bash"), execute_cmd)
          )
        end

        # 4- Move old nps directory to tomcat5/nps_2.0.2
        execute_cmd = "cp -r /var/opt/novell/iManager/nps /var/opt/novell/tomcat4"
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval != 0
          Report.Warning(
            _(
              "Unable to copy old iManager nps directory under the tomcat4 directory."
            )
          )
        end

        # (Temp) Rename the old nps to nps_<version>
        version_number = Ops.add("/var/opt/novell/tomcat4/nps_", iman_version)
        execute_cmd = Ops.add("mv /var/opt/novell/tomcat4/nps ", version_number)
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval != 0
          Report.Warning(
            _(
              "Unable to rename old iManager nps directory under the tomcat4 directory."
            )
          )
        end

        # Delete the old IManager directories before we do the upgrade
        execute_cmd = "rm -r /var/opt/novell/iManager/nps"
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval != 0
          Report.Warning(_("Unable to remove old iManager nps directory."))
        end
      end

      #Store the Tree CA certificate to the Java Key Store
      Builtins.y2milestone(
        "iManager:Storing the Tree CA Root Certificate to the Java keystore"
      )
      treeName = "unknown" if Builtins.size(treeName) == 0

      _alias = Ops.add(Ops.add("iManager_", treeName), "_sscert")

      # Novell's Root store  /var/opt/novell/tomcat/conf/cacerts"
      rootCertStore = "/var/opt/novell/tomcat/conf/cacerts"

      stored = NovellUtils.StoreTrustedCertificateInJavaKeystore(
        "/etc/opt/novell/certs/SSCert.der",
        rootCertStore,
        _alias
      )
      if !stored
        message = "Unable to add the tree self-signed certificate to the root certificate store"
        Builtins.y2milestone("iManager: %1", message)
      else
        Builtins.y2milestone(
          "iManager: Tree CA Root Certificate was stored in the Novell keystore"
        )
      end

      # Note: Not sure if we need this with cypress. The edir libraries are installed by someone else.
      # Set Tomcat LD_LIBRARY_PATH to locate the edir 8.8 shared libraries
      #execute_cmd = "/opt/novell/iManager/bin/set_libpath.sh";
      #execute_cmd = "LD_LIBRARY_PATH=/opt/novell/eDirectory/lib";
      #   y2milestone("Executing %1", execute_cmd);
      #string libPath = "/opt/novell/eDirectory/lib";
      #execute_cmd = "$[LD_LIBRARY_PATH:libPath]";
      #rval = (integer)SCR::Execute(.target.bash, execute_cmd);
      #if(rval != 0)
      #   Report::Warning(_("Unable to set LD_LIBRARY_PATH."));
      #rmap = (map)SCR::Execute(.target.bash_output, cmd);


      # 5 - At this point we need to check if we are upgrading from a previous version of iManager
      # If we are, we need to copy over the npms and custom code from the previous install before we install
      if upgrade_iman == "yes"
        # Check for field patched and move them so we dont install them with 2.5
        execute_cmd = Ops.add(
          Ops.add(
            Ops.add("mv /var/opt/novell/tomcat4/nps_", iman_version),
            "/packages/iMan202_*.npm /var/opt/novell/tomcat4/nps_"
          ),
          iman_version
        )
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )

        # Copy over existing npm files -- but only if they are newer than 2.5
        execute_cmd = Ops.add(
          Ops.add("cp -ru /var/opt/novell/tomcat4/nps_", iman_version),
          "/packages/* /var/opt/novell/tomcat5/webapps/nps/packages/"
        )
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval != 0
          Report.Warning(
            _("Unable to copy old npms from previous version of iManager")
          )
        end

        # move field patches back into nps_version directory
        execute_cmd = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add("mv /var/opt/novell/tomcat4/nps_", iman_version),
              "/iMan202_*.npm /var/opt/novell/tomcat5/nps_"
            ),
            iman_version
          ),
          "/packages/"
        )
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )

        # copy over any custom code from the previous installation
        execute_cmd = Ops.add(
          Ops.add("cp -ru /var/opt/novell/tomcat4/nps_", iman_version),
          "/portal/modules/custom/* /var/opt/novell/tomcat5/webapps/nps/portal/modules/custom/"
        )
        rval = Convert.to_integer(
          SCR.Execute(path(".target.bash"), execute_cmd)
        )
        if rval != 0
          Report.Warning(
            _("Unable to copy custom code from previous version of iManager")
          )
        end
      end

      # Call imanager_installNPM.sh to install the npms. The only parameter we need to pass here is the 'install' param.
      return false if Abort()
      Progress.NextStage

      do_plugins = Ops.get_string(
        @Settings,
        "CONFIG_IMANAGER_INSTALL_PLUGINS",
        "yes"
      )
      doPlugins = true
      doPlugins = false if do_plugins == "no"

      # Put message here as temp location to get it translated.
      pluginQuestion = _(
        "Do you want to install iManager plugins now?\n" +
          "\n" +
          "Installing plugins during this install will take several additional minutes.\n" +
          "After installation, you can install plugins from iManager, using the 'Available Novell Plug-in Modules' task."
      )
      # end temp message

      if doPlugins
        Builtins.y2milestone("mtc:doing plugins.")
        Popup.ShowFeedback(
          _("Please Wait"),
          _("Installing iManager Plug-ins. This may take a while...")
        )
        installNPMs("install")
        Popup.ClearFeedback
      else
        Builtins.y2milestone("mtc:skipping plugins.")
      end

      # Changing owner and user for Cypress OES from novlwww:www to wwwrun:www
      execute_cmd = "chown -R wwwrun:www /var/opt/novell/iManager/"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      if rval != 0
        Report.Warning(
          _("Unable to change file owner to wwwrun and group to www.")
        )
      end

      # Note: This section was used for the set_libpath.sh and set_libpath.sed files
      # Changing owner and user for Cypress OES from novlwww:www to wwwrun:www
      #execute_cmd = "chown -R wwwrun:www /opt/novell/iManager/";
      #rval = (integer)SCR::Execute(.target.bash, execute_cmd);
      #if(rval != 0)
      #   Report::Warning(_("Unable to change file owner to wwwrun and group to www."));

      # Changing owner and user for Cypress OES from novlwww:www to wwwrun:www
      # Change permissions to www user in order to run pre-compiled jsps.
      # Note: For oes we probably need to set this to a different directory under work.
      # Note: commented out this section since we no longer create this directory
      #string WORK_NPS = "/var/opt/novell/tomcat/work";
      #execute_cmd = "chown -R wwwrun:www " + WORK_NPS;
      #rval = (integer)SCR::Execute(.target.bash, execute_cmd);
      #if(rval != 0)
      #   Report::Warning(_("Unable to change /var/opt/novell/tomcat/work/Standalone/localhost/nps permissions."));

      # Changing owner and user for Cypress OES from novlwww:www to wwwrun:www
      execute_cmd = "chown -R wwwrun:www /var/opt/novell/tomcat/webapps/nps/"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      if rval != 0
        Report.Warning(
          _("Unable to change /var/opt/novell/tomcat/webapps/nps permissions.")
        )
      end

      execute_cmd = "chown -R wwwrun:www /var/opt/novell/tomcat/work/Catalina"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      if rval != 0
        Builtins.y2milestone(
          "Unable to change /var/opt/novell/tomcat/work/Catalina permissions."
        )
      end

      # *****************************************************************************
      #  * The following command will only work on ENGLIGH systems. Can you
      #  * see why??
      #  ******************************************************************************
      #     execute_cmd = "echo -e 'changeit\nyes' | /opt/novell/lib/java2/bin/keytool -import -file /etc/ssl/servercerts/servercert.pem -alias `hostname` -keystore " + rootCertStore;
      #     **************************************************************************

      #**********************************************************************
      # Store the server certificate that was generated by the YaST CA for
      # use by Apache........
      #***********************************************************************
      _alias = NovellUtils.GetHostname
      stored = NovellUtils.StoreTrustedCertificateInJavaKeystore(
        "/etc/ssl/servercerts/servercert.pem",
        rootCertStore,
        _alias
      )

      if !stored
        Builtins.y2milestone(
          "iManager: Store of server certificate to Novell Java keystore failed"
        ) 
        # TODO: Report an Error HERE when translation of strings opens up again
      else
        Builtins.y2milestone(
          "iManager: Store of server certificate to Novell Java keystore was successful"
        )
      end

      if Mode.normal
	if @channelUpgradeMode != true
          if Popup.YesNo(
              _(
                "iManager needs to restart Tomcat and Apache.\nDo you want to restart them now?"
              )
            )
            # Bug:97825 switched apache to stop before tomcat is restarted and start after tomcat is restarted.
            # apache would sometimes fail to restart on an OES SP1 patched system because apache was not
            # completely stopped before it was trying to be restarted.
            Service.Stop("apache2")
            Service.Stop("novell-tomcat")
            Service.Start("novell-tomcat")
            Service.Start("apache2") 
            #Service::Restart("novell-tomcat");
            #Service::Restart("apache2");
          end
	else
	    Service.Stop("apache2")
	    Service.Stop("novell-tomcat")
	    Service.Start("novell-tomcat")
	    Service.Start("apache2")
	end
      else
        SCR.Write(path(".sysconfig.novell-common.START_APACHE_TOMCAT"), "yes")
      end

      # open ports 80 and 443 in the firewall
      ConfigureFirewall()

      return false if Abort()
      # Progress finished
      Progress.Finish
      Builtins.sleep(sl)

      return false if Abort()

      SCR.Write(
        path(".sysconfig.imanager.CONFIG_IMANAGER_INSTALL_PLUGINS"),
        do_plugins
      )
      SCR.Write(path(".sysconfig.imanager.SERVICE_CONFIGURED"), "yes")

      # Flush the sysconfig file.
      SCR.Write(path(".sysconfig.imanager"), nil)


      true
    end

    # Get all imanager settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "imanager", false)
        return true
      end

      Ops.set(
        @Settings,
        "CONFIG_IMANAGER_INSTALL_PLUGINS",
        Ops.get_string(settings, "install_plugins", "yes")
      )

      @globals_configured = true
      true
    end

    # Dump the imanager settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-imanager") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "imanager", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "imanager", true)
        Ops.set(settings, "configure_now", true)
        Ops.set(
          settings,
          "install_plugins",
          Ops.get_string(@Settings, "CONFIG_IMANAGER_INSTALL_PLUGINS", "yes")
        )
      end
      deep_copy(settings)
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      end
      # Bug 291964 - Don't show tree name and user name since they are not required
      #else
      #{
      #   summary = Summary::OpenList(summary);
      #
      #   summary = Summary::AddListItem(
      #  summary, sformat(_("eDirectory tree name: %1"),
      #		   NovellLDAP::Settings["CONFIG_LDAP_TREE_NAME"]:"") );
      #
      #   summary = Summary::AddListItem(
      #  summary, sformat(_("eDirectory admin name with context: %1"),
      #		   NovellLDAP::Settings["CONFIG_LDAP_ADMIN_CONTEXT"]:"") );
      #
      #   summary = Summary::CloseList(summary);
      #}
      #
      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      { "install" => [], "remove" => [] }
    end

    # Determine the iManager rpm version
    #
    # @return [String] with iManager rpm version.
    def GetImanagerVersion
      iman_version = "1.5"

      execute_cmd = "rpm -q novell-imanager-2.7"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      if rval == 0
        iman_version = "2.7"
        return iman_version
      end

      execute_cmd = "rpm -q novell-imanager-2.6"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      if rval == 0
        iman_version = "2.6"
        return iman_version
      end

      execute_cmd = "rpm -q novell-imanager-2.5"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      if rval == 0
        iman_version = "2.5"
        return iman_version
      end

      execute_cmd = "rpm -q novell-imanager-2.0.2"
      rval = Convert.to_integer(SCR.Execute(path(".target.bash"), execute_cmd))
      if rval == 0
        iman_version = "2.0.2"
        return iman_version
      end

      iman_version
    end

    publish :variable => :auto_clone, :type => "boolean"
    publish :variable => :xad_packages_installed, :type => "boolean"
    publish :variable => :xadMode, :type => "boolean"
    publish :variable => :xadTestMode, :type => "boolean"
    publish :variable => :Settings, :type => "map"
    publish :variable => :oldSettings, :type => "map"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :upgradeModeOES11SP2, :type => "boolean"
    publish :variable => :upgradeModeOES11SP3, :type => "boolean"
    publish :variable => :upgradeModeOES2015, :type => "boolean"
    publish :variable => :upgradeModeOES2015SP1, :type => "boolean"
    publish :variable => :upgradeModeOES2018, :type => "boolean"
    publish :variable => :upgradeModeOES2018SP1, :type => "boolean"
    publish :variable => :upgradeModeOES2018SP2, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :installNPMs, :type => "boolean (string)"
    publish :function => :uninstall_iman202, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
    publish :variable => :channelUpgradeMode, :type => "boolean"
  end

  Imanager = ImanagerClass.new
  Imanager.main
end
